import importlib
import threading
from unittest import TestSuite, TestLoader

from xmlrunner import xmlrunner

from ruthie.toolset import discoverer
from .base import Base


class Parallel(Base):
    threads = None
    queue = None

    def run(self):
        assert self.options['--threads'].isdigit(), "Threads must be a number"

        self.threads = int(self.options['--threads'])
        assert self.threads > 1, "Threads number must be greater than 1"

        self.queue = discoverer.classes_in(self.options['<path>'])

        for i in range(self.threads):
            t = threading.Thread(target=self.run_tests)
            t.start()

    def run_tests(self):
        while len(self.queue) > 0:
            print("Taking from queue...")
            test_params = self.queue.pop()

            print(f"Importing {test_params[0]}")
            module_object = importlib.import_module(test_params[0])
            test_class = getattr(module_object, test_params[1])
            loader = TestLoader()
            suite = TestSuite(
                (
                    loader.loadTestsFromTestCase(test_class)
                )
            )

            print(f"Running {test_params[0]}.{test_params[1]}")
            runner = xmlrunner.XMLTestRunner(output="junit", verbosity=2)
            runner.run(suite)



#!/usr/bin/env python3

"""
ruthie

Run Unit Tests Harmoniously Incredibly Easy

Usage:
  ruthie parallel [--threads=<threads>] <path>
  ruthie discover classes [--group] <path>
  ruthie -h | --help
  ruthie --version

Commands:
  parallel                                  Discover all test classes in given path and run them in parallel
  discover classes                          Discover all test classes in given path and print them in the console

Options:
  -t <threads> --threads=<threads>          Number of parallel threads [default: 2]
  -g --group                                Group results [default: False]
  -s --skip-cleanup                         Skip cleanup of results when running on prodbuild
  -h --help                                 Show this screen
  --version                                 Show version

Examples:
  ruthie parallel unittests
"""


from inspect import getmembers, isclass
from docopt import docopt
from ruthie import __version__ as version
from ruthie import commands


def main():
    """Main CLI entrypoint."""
    options = docopt(__doc__, version=version)

    # Here we'll try to dynamically match the command the user is trying to run
    # with a pre-defined command class we've already created.
    for (k, v) in list(options.items()):
        _k = k.replace('-', '_')
        if hasattr(commands, _k) and v:
            module = getattr(commands, _k)
            _commands = getmembers(module, isclass)
            command = [command[1] for command in _commands if command[0] != 'Base'][0]
            command = command(options)
            command.run()



import os
import re


def get_file_path(unittests_path):
    for unittests_root, unittests_dirs, unittests_files in os.walk(unittests_path):
        module_root = unittests_root.replace(os.path.sep, '.')

        for file_name in unittests_files:
            if file_name[0] in ['.', '_']:
                continue

            module_name, file_ext = os.path.splitext(file_name)
            if file_ext != '.py':
                continue

            yield f'{module_root}.{module_name}'

        for dir_name in unittests_dirs:
            yield from get_file_path(f'{module_root}.{dir_name}')


def get_file_line(unittests_path):
    gen_get_file_path = get_file_path(unittests_path)
    for module_path in gen_get_file_path:
        module_file_path = module_path.replace('.', os.path.sep) + '.py'
        with(open(module_file_path, 'r')) as file:
            while True:
                line = file.readline()
                if not line:
                    break

                yield module_path, line


def classes_in(unittests_path):
    classes = []
    current_class = None
    test_found = False

    gen_get_file_line = get_file_line(unittests_path)
    for module_path, line in gen_get_file_line:
        class_name = re.match(r"class ([^(]+)", line)
        if class_name:
            current_class = class_name.group(1)
            test_found = False
            continue

        if not test_found and re.match(r" +def test_", line):
            classes.append((module_path, current_class))
            test_found = True

    return classes



from .base import Base
from ruthie.toolset import discoverer


class Discover(Base):

    def run(self):
        if self.options['classes']:
            if self.options['--group']:
                self.print_classes_grouped()
            else:
                self.print_classes_listed()
        else:
            raise AttributeError('Unsupported command')

    def print_classes_grouped(self):
        last = None
        for discovery in discoverer.classes_in(self.options['<path>']):
            if last != discovery[0]:
                print(f'  {discovery[0]}')
                last = discovery[0]
            print(f'    {discovery[1]}')

    def print_classes_listed(self):
        for discovery in discoverer.classes_in(self.options['<path>']):
            print(f'{discovery[0]}.{discovery[1]}')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ruthie', 'ruthie.commands', 'ruthie.toolset']

package_data = \
{'': ['*']}

install_requires = \
['docopt==0.6.2', 'toml>=0.9,<0.10', 'xmlrunner>=1.7.7,<1.8.0']

entry_points = \
{'console_scripts': ['ruthie = =ruthie.cli:main']}

setup_kwargs = {
    'name': 'ruthie',
    'version': '0.1.1',
    'description': 'Run Unit Tests Harmoniously Incredibly Easy',
    'long_description': '# ruthie\n\n[![Current Release](https://img.shields.io/github/release/bitbar/ruthie.svg)](releases)\n[![License: ISC](https://img.shields.io/badge/License-ISC-blue.svg)](LICENSE.md)\n\nRun Unit Tests Harmoniously Incredibly Easy\n\nThis repo contains _ruthie_ which is the Unittests runner.\n\n## Installation\n\nTODO\n\n## Usage\n\nTODO\n\n## License\n\nThis project is licensed under the ISC License - see the [LICENSE](LICENSE) file for details.\n',
    'author': 'Marek Sierociński',
    'author_email': 'marek.sierocinski@smartbear.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/bitbar/ruthie',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
